## $Id: test_document.py 12205 2014-12-12 22:14:37Z henrik $
##
## Copyright (C) 2014 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Tests for documents.
"""
import shutil
import tempfile
import grok
from StringIO import StringIO
from zope.interface.verify import verifyClass, verifyObject
from zope.component import createObject, queryUtility
from hurry.workflow.interfaces import (
    IWorkflowInfo, IWorkflowState, InvalidTransitionError)
from waeup.ikoba.interfaces import (
    IObjectHistory, IFileStoreNameChooser, IFileStoreHandler)
from waeup.ikoba.imagestorage import DefaultStorage
from waeup.ikoba.customers.interfaces import (
    ICustomerDocumentsContainer, ICustomerDocument)
from waeup.ikoba.interfaces import IObjectHistory
from waeup.ikoba.customers.customer import Customer
from waeup.ikoba.customers.documents import (
    CustomerDocumentsContainer, CustomerSampleDocument,
    CustomerDocumentFileStoreHandler, CustomerDocumentFileNameChooser)
from waeup.ikoba.testing import (FunctionalLayer, FunctionalTestCase)

class CustomerDocumentsContainerTestCase(FunctionalTestCase):

    layer = FunctionalLayer

    def test_interfaces(self):
        # Make sure the correct interfaces are implemented.
        self.assertTrue(
            verifyClass(
                ICustomerDocumentsContainer, CustomerDocumentsContainer)
            )
        self.assertTrue(
            verifyObject(
                ICustomerDocumentsContainer, CustomerDocumentsContainer())
            )
        self.assertTrue(
            verifyClass(
                ICustomerDocument, CustomerSampleDocument)
            )
        self.assertTrue(
            verifyObject(
                ICustomerDocument, CustomerSampleDocument())
            )
        return

    def test_addDocument(self):
        container = CustomerDocumentsContainer()
        document = createObject(u'waeup.CustomerSampleDocument')
        id = document.document_id
        container.addDocument(document)
        self.assertEqual(container[id], document)
        self.assertRaises(TypeError, container.addDocument, object())
        self.assertEqual(document.class_name, 'CustomerSampleDocument')
        return

    def test_document_workflow(self):
        document = createObject(u'waeup.CustomerSampleDocument')
        IWorkflowInfo(document).fireTransition('create')
        self.assertEqual(IWorkflowState(document).getState(), 'created')
        IWorkflowInfo(document).fireTransition('submit')
        self.assertEqual(IWorkflowState(document).getState(), 'submitted')
        IWorkflowInfo(document).fireTransition('verify')
        self.assertEqual(IWorkflowState(document).getState(), 'verified')
        IWorkflowInfo(document).fireTransition('reset2')
        self.assertEqual(IWorkflowState(document).getState(), 'created')
        self.assertRaises(InvalidTransitionError,
            IWorkflowInfo(document).fireTransition, 'verify')
        IWorkflowInfo(document).fireTransition('submit')
        IWorkflowInfo(document).fireTransition('reset3')
        self.assertEqual(IWorkflowState(document).getState(), 'created')
        return

    def test_document_history(self):
        doc = createObject(u'waeup.CustomerSampleDocument')
        IWorkflowInfo(doc).fireTransition('create')
        messages = ' '.join(doc.history.messages)
        self.assertTrue('Document created by system' in messages)
        return


class CustomerDocumentFileTests(FunctionalTestCase):

    layer = FunctionalLayer

    def setUp(self):
        super(CustomerDocumentFileTests, self).setUp()
        self.workdir = tempfile.mkdtemp()
        return

    def tearDown(self):
        super(CustomerDocumentFileTests, self).tearDown()
        shutil.rmtree(self.workdir)
        return

    def test_file_store_handler_util_accessible(self):
        # we can get an IFileStoreHandler utility for documents
        handler = queryUtility(IFileStoreHandler, name='file-customerdocument')
        self.assertTrue(
            isinstance(handler, CustomerDocumentFileStoreHandler))
        return

    def test_file_store_handler(self):
        store = DefaultStorage()
        handler = queryUtility(IFileStoreHandler, name='file-customerdocument')
        result = handler.pathFromFileID(
            store, '/fake-root', '__anything__345/c999/nice_image_d123_c999.jpeg')
        self.assertEqual(
            result, '/fake-root/customers/345/c999/nice_image_d123_c999')
        return

    def test_file_store_handler_create(self):
        # we can create files in image store with the document file
        # store handler
        store = DefaultStorage(self.workdir)
        handler = queryUtility(IFileStoreHandler, name='file-customerdocument')
        file_obj, path, waeup_file = handler.createFile(
            store, store.root, 'sample.jpg', '__anything__345/c999/nice_image_d123_c999.jpeg',
            StringIO('I am a JPG file'))
        self.assertEqual(path[-43:], 'customers/345/c999/nice_image_d123_c999.jpg')
        return

    def test_iface(self):
        # make sure we implement promised interfaces
        obj = CustomerDocumentFileNameChooser(None) # needs a context normally
        verifyClass(IFileStoreNameChooser, CustomerDocumentFileNameChooser)
        verifyObject(IFileStoreNameChooser, obj)
        return

    def test_name_chooser_available(self):
        # we can get a name chooser for document objects as adapter
        doc = CustomerSampleDocument()
        chooser = IFileStoreNameChooser(doc)
        self.assertTrue(chooser is not None)
        return

    def test_name_chooser_document(self):
        # we can't get an image filename for documents not owned by a customer
        doc = CustomerSampleDocument()
        chooser = IFileStoreNameChooser(doc)
        # no customer can be found
        self.assertRaises(AttributeError, chooser.chooseName, 'sample.jpg')
        # we add a fake customer and fake documents container and can get
        # an image name
        fake_customer = Customer()
        fake_customer.customer_id = u'K1000001'
        fake_container = grok.Container()
        fake_container.__parent__ = fake_customer
        doc.__parent__ = fake_container
        result = chooser.chooseName('sample.jpg')
        self.assertEqual(
            result, '__file-customerdocument__01000/K1000001/sample_d123_K1000001.jpg')
        return